@@ -86,7 +86,7 @@ __midentify__allprops="
     video-bitrate
     video-codec
     video-format
-    aspect
+    video-aspect
     fps
     width
     height
@@ -104,7 +104,7 @@ for __midentify__key in $__midentify__allprops; do
     eval unset $__midentify__nextprefix$__midentify__key
 done
 
-__midentify__output=`$MPV --playing-msg="$__midentify__propstr" --vo=null --ao=null --frames=1 --quiet --no-cache --no-config "$@"`
+__midentify__output=`$MPV --term-playing-msg="$__midentify__propstr" --vo=null --ao=null --frames=1 --quiet --no-cache --no-config "$@"`
 __midentify__fileindex=0
 __midentify__prefix=
 while :; do
