@@ -38,7 +38,7 @@
 #include "config.h"
 #include "audio/format.h"
 #include "ao.h"
-#include "audio/reorder_ch.h"
+#include "internal.h"
 #include "common/msg.h"
 #include "osdep/timer.h"
 #include "options/m_option.h"
@@ -300,23 +300,12 @@ static int write_buffer(struct ao *ao, unsigned char *data, int len)
 
 
     if (SUCCEEDED(res)) {
-        if (!AF_FORMAT_IS_AC3(ao->format)) {
-            memcpy(lpvPtr1, data, dwBytes1);
-            if (lpvPtr2 != NULL)
-                memcpy(lpvPtr2, (char *)data + dwBytes1, dwBytes2);
-
-            p->write_offset += dwBytes1 + dwBytes2;
-            if (p->write_offset >= p->buffer_size)
-                p->write_offset = dwBytes2;
-        } else {
-            // Write to pointers without reordering.
-            memcpy(lpvPtr1, data, dwBytes1);
-            if (NULL != lpvPtr2)
-                memcpy(lpvPtr2, data + dwBytes1, dwBytes2);
-            p->write_offset += dwBytes1 + dwBytes2;
-            if (p->write_offset >= p->buffer_size)
-                p->write_offset = dwBytes2;
-        }
+        memcpy(lpvPtr1, data, dwBytes1);
+        if (NULL != lpvPtr2)
+            memcpy(lpvPtr2, data + dwBytes1, dwBytes2);
+        p->write_offset += dwBytes1 + dwBytes2;
+        if (p->write_offset >= p->buffer_size)
+            p->write_offset = dwBytes2;
 
         // Release the data back to DirectSound.
         res = IDirectSoundBuffer_Unlock(p->hdsbuf, lpvPtr1, dwBytes1, lpvPtr2,
@@ -413,7 +402,7 @@ static int init(struct ao *ao)
     //set our audio parameters
     ao->samplerate = rate;
     ao->format = format;
-    ao->bps = ao->channels.num * rate * (af_fmt2bits(format) >> 3);
+    ao->bps = ao->channels.num * rate * af_fmt2bps(format);
     int buffersize = ao->bps; // space for 1 sec
     MP_VERBOSE(ao, "Samplerate:%iHz Channels:%i Format:%s\n", rate,
                ao->channels.num, af_fmt_to_str(format));
@@ -433,9 +422,9 @@ static int init(struct ao *ao)
     } else {
         wformat.Format.wFormatTag = (ao->channels.num > 2)
                                     ? WAVE_FORMAT_EXTENSIBLE : WAVE_FORMAT_PCM;
-        wformat.Format.wBitsPerSample = af_fmt2bits(format);
-        wformat.Format.nBlockAlign = wformat.Format.nChannels *
-                                     (wformat.Format.wBitsPerSample >> 3);
+        int bps = af_fmt2bps(format);
+        wformat.Format.wBitsPerSample = bps * 8;
+        wformat.Format.nBlockAlign = wformat.Format.nChannels * bps;
     }
 
     // fill in primary sound buffer descriptor
@@ -545,10 +534,8 @@ static void audio_resume(struct ao *ao)
 \brief close audio device
 \param immed stop playback immediately
 */
-static void uninit(struct ao *ao, bool immed)
+static void uninit(struct ao *ao)
 {
-    if (!immed)
-        mp_sleep_us(get_delay(ao) * 1000000);
     reset(ao);
 
     DestroyBuffer(ao);
