@@ -25,13 +25,45 @@
 #include "encode_lavc.h"
 #include "common/global.h"
 #include "common/msg.h"
+#include "common/msg_control.h"
 #include "video/vfcap.h"
+#include "options/m_option.h"
 #include "options/options.h"
 #include "osdep/timer.h"
 #include "video/out/vo.h"
 #include "talloc.h"
 #include "stream/stream.h"
 
+#define OPT_BASE_STRUCT struct encode_opts
+const struct m_sub_options encode_config = {
+    .opts = (const m_option_t[]) {
+        OPT_STRING("o", file, CONF_GLOBAL | CONF_NOCFG | CONF_PRE_PARSE),
+        OPT_STRING("of", format, CONF_GLOBAL),
+        OPT_STRINGLIST("ofopts*", fopts, CONF_GLOBAL),
+        OPT_FLOATRANGE("ofps", fps, CONF_GLOBAL, 0.0, 1000000.0),
+        OPT_FLOATRANGE("omaxfps", maxfps, CONF_GLOBAL, 0.0, 1000000.0),
+        OPT_STRING("ovc", vcodec, CONF_GLOBAL),
+        OPT_STRINGLIST("ovcopts*", vopts, CONF_GLOBAL),
+        OPT_STRING("oac", acodec, CONF_GLOBAL),
+        OPT_STRINGLIST("oacopts*", aopts, CONF_GLOBAL),
+        OPT_FLAG("oharddup", harddup, CONF_GLOBAL),
+        OPT_FLOATRANGE("ovoffset", voffset, CONF_GLOBAL, -1000000.0, 1000000.0),
+        OPT_FLOATRANGE("oaoffset", aoffset, CONF_GLOBAL, -1000000.0, 1000000.0),
+        OPT_FLAG("ocopyts", copyts, CONF_GLOBAL),
+        OPT_FLAG("orawts", rawts, CONF_GLOBAL),
+        OPT_FLAG("oautofps", autofps, CONF_GLOBAL),
+        OPT_FLAG("oneverdrop", neverdrop, CONF_GLOBAL),
+        OPT_FLAG("ovfirst", video_first, CONF_GLOBAL),
+        OPT_FLAG("oafirst", audio_first, CONF_GLOBAL),
+        OPT_FLAG("ometadata", metadata, CONF_GLOBAL),
+        {0}
+    },
+    .size = sizeof(struct encode_opts),
+    .defaults = &(const struct encode_opts){
+        .metadata = 1,
+    },
+};
+
 static int set_to_avdictionary(struct encode_lavc_context *ctx,
                                AVDictionary **dictp,
                                const char *key,
@@ -102,6 +134,15 @@ static bool value_has_flag(const char *value, const char *flag)
         return val; \
     }
 
+#define CHECK_FAIL_UNLOCK(ctx, val) \
+    if (ctx && (ctx->failed || ctx->finished)) { \
+        MP_ERR(ctx, \
+               "Called a function on a %s encoding context. Bailing out.\n", \
+               ctx->failed ? "failed" : "finished"); \
+        pthread_mutex_unlock(&ctx->lock); \
+        return val; \
+    }
+
 int encode_lavc_available(struct encode_lavc_context *ctx)
 {
     CHECK_FAIL(ctx, 0);
@@ -114,7 +155,7 @@ int encode_lavc_oformat_flags(struct encode_lavc_context *ctx)
     return ctx->avc ? ctx->avc->oformat->flags : 0;
 }
 
-struct encode_lavc_context *encode_lavc_init(struct encode_output_conf *options,
+struct encode_lavc_context *encode_lavc_init(struct encode_opts *options,
                                              struct mpv_global *global)
 {
     struct encode_lavc_context *ctx;
@@ -133,6 +174,7 @@ struct encode_lavc_context *encode_lavc_init(struct encode_output_conf *options,
         mp_msg_force_stderr(global, true);
 
     ctx = talloc_zero(NULL, struct encode_lavc_context);
+    pthread_mutex_init(&ctx->lock, NULL);
     ctx->log = mp_log_new(ctx, global->log, "encode-lavc");
     ctx->global = global;
     encode_lavc_discontinuity(ctx);
@@ -233,10 +275,16 @@ struct encode_lavc_context *encode_lavc_init(struct encode_output_conf *options,
     return ctx;
 }
 
+void encode_lavc_set_metadata(struct encode_lavc_context *ctx,
+                              struct mp_tags *metadata)
+{
+    if (ctx->options->metadata)
+        ctx->metadata = metadata;
+}
+
 int encode_lavc_start(struct encode_lavc_context *ctx)
 {
     AVDictionaryEntry *de;
-    unsigned i;
 
     if (ctx->header_written < 0)
         return 0;
@@ -246,23 +294,31 @@ int encode_lavc_start(struct encode_lavc_context *ctx)
     CHECK_FAIL(ctx, 0);
 
     if (ctx->expect_video) {
+        unsigned i;
         for (i = 0; i < ctx->avc->nb_streams; ++i)
             if (ctx->avc->streams[i]->codec->codec_type == AVMEDIA_TYPE_VIDEO)
                 break;
         if (i >= ctx->avc->nb_streams) {
-            encode_lavc_fail(ctx,
-                    "video stream missing, invalid codec?\n");
-            return 0;
+            if (ctx->avc->oformat->video_codec != AV_CODEC_ID_NONE ||
+                ctx->options->vcodec) {
+                encode_lavc_fail(ctx,
+                    "no video stream succeeded - invalid codec?\n");
+                return 0;
+            }
         }
     }
     if (ctx->expect_audio) {
+        unsigned i;
         for (i = 0; i < ctx->avc->nb_streams; ++i)
             if (ctx->avc->streams[i]->codec->codec_type == AVMEDIA_TYPE_AUDIO)
                 break;
         if (i >= ctx->avc->nb_streams) {
-            encode_lavc_fail(ctx,
-                    "audio stream missing, invalid codec?\n");
-            return 0;
+            if (ctx->avc->oformat->audio_codec != AV_CODEC_ID_NONE ||
+                ctx->options->acodec) {
+                encode_lavc_fail(ctx,
+                    "no audio stream succeeded - invalid codec?\n");
+                return 0;
+            }
         }
     }
 
@@ -285,6 +341,13 @@ int encode_lavc_start(struct encode_lavc_context *ctx)
     MP_INFO(ctx, "Opening muxer: %s [%s]\n",
             ctx->avc->oformat->long_name, ctx->avc->oformat->name);
 
+    if (ctx->metadata) {
+        for (int i = 0; i < ctx->metadata->num_keys; i++) {
+            av_dict_set(&ctx->avc->metadata,
+                ctx->metadata->keys[i], ctx->metadata->values[i], 0);
+        }
+    }
+
     if (avformat_write_header(ctx->avc, &ctx->foptions) < 0) {
         encode_lavc_fail(ctx, "could not write header\n");
         return 0;
@@ -308,6 +371,7 @@ void encode_lavc_free(struct encode_lavc_context *ctx)
         encode_lavc_fail(ctx,
                          "called encode_lavc_free without encode_lavc_finish\n");
 
+    pthread_mutex_destroy(&ctx->lock);
     talloc_free(ctx);
 }
 
@@ -382,7 +446,19 @@ void encode_lavc_finish(struct encode_lavc_context *ctx)
 
 void encode_lavc_set_video_fps(struct encode_lavc_context *ctx, float fps)
 {
+    pthread_mutex_lock(&ctx->lock);
     ctx->vo_fps = fps;
+    pthread_mutex_unlock(&ctx->lock);
+}
+
+void encode_lavc_set_audio_pts(struct encode_lavc_context *ctx, double pts)
+{
+    if (ctx) {
+        pthread_mutex_lock(&ctx->lock);
+        ctx->last_audio_in_pts = pts;
+        ctx->samples_since_last_pts = 0;
+        pthread_mutex_unlock(&ctx->lock);
+    }
 }
 
 static void encode_2pass_prepare(struct encode_lavc_context *ctx,
@@ -506,7 +582,10 @@ AVStream *encode_lavc_alloc_stream(struct encode_lavc_context *ctx,
     switch (mt) {
     case AVMEDIA_TYPE_VIDEO:
         if (!ctx->vc) {
-            encode_lavc_fail(ctx, "vo-lavc: encoder not found\n");
+            if (ctx->avc->oformat->video_codec != AV_CODEC_ID_NONE ||
+                ctx->options->vcodec) {
+                encode_lavc_fail(ctx, "vo-lavc: encoder not found\n");
+            }
             return NULL;
         }
         avcodec_get_context_defaults3(stream->codec, ctx->vc);
@@ -539,7 +618,10 @@ AVStream *encode_lavc_alloc_stream(struct encode_lavc_context *ctx,
 
     case AVMEDIA_TYPE_AUDIO:
         if (!ctx->ac) {
-            encode_lavc_fail(ctx, "ao-lavc: encoder not found\n");
+            if (ctx->avc->oformat->audio_codec != AV_CODEC_ID_NONE ||
+                ctx->options->acodec) {
+                encode_lavc_fail(ctx, "ao-lavc: encoder not found\n");
+            }
             return NULL;
         }
         avcodec_get_context_defaults3(stream->codec, ctx->ac);
@@ -771,11 +853,15 @@ void encode_lavc_discontinuity(struct encode_lavc_context *ctx)
     if (!ctx)
         return;
 
-    CHECK_FAIL(ctx, );
+    pthread_mutex_lock(&ctx->lock);
+
+    CHECK_FAIL_UNLOCK(ctx, );
 
     ctx->audio_pts_offset = MP_NOPTS_VALUE;
     ctx->last_video_in_pts = MP_NOPTS_VALUE;
     ctx->discontinuity_pts_offset = MP_NOPTS_VALUE;
+
+    pthread_mutex_unlock(&ctx->lock);
 }
 
 static void encode_lavc_printoptions(struct mp_log *log, void *obj,
@@ -850,7 +936,7 @@ static void encode_lavc_printoptions(struct mp_log *log, void *obj,
     }
 }
 
-bool encode_lavc_showhelp(struct mp_log *log, struct encode_output_conf *opts)
+bool encode_lavc_showhelp(struct mp_log *log, struct encode_opts *opts)
 {
     bool help_output = false;
     if (av_codec_next(NULL) == NULL)
@@ -1004,7 +1090,9 @@ int encode_lavc_getstatus(struct encode_lavc_context *ctx,
     if (!ctx)
         return -1;
 
-    CHECK_FAIL(ctx, -1);
+    pthread_mutex_lock(&ctx->lock);
+
+    CHECK_FAIL_UNLOCK(ctx, -1);
 
     minutes = (now - ctx->t0) / 60.0 * (1 - f) / f;
     megabytes = ctx->avc->pb ? (avio_size(ctx->avc->pb) / 1048576.0 / f) : 0;
@@ -1020,12 +1108,16 @@ int encode_lavc_getstatus(struct encode_lavc_context *ctx,
         snprintf(buf, bufsize, "{%.1fmin %.1fMB}",
                  minutes, megabytes);
     buf[bufsize - 1] = 0;
+
+    pthread_mutex_unlock(&ctx->lock);
     return 0;
 }
 
 void encode_lavc_expect_stream(struct encode_lavc_context *ctx, int mt)
 {
-    CHECK_FAIL(ctx, );
+    pthread_mutex_lock(&ctx->lock);
+
+    CHECK_FAIL_UNLOCK(ctx, );
 
     switch (mt) {
     case AVMEDIA_TYPE_VIDEO:
@@ -1035,11 +1127,18 @@ void encode_lavc_expect_stream(struct encode_lavc_context *ctx, int mt)
         ctx->expect_audio = true;
         break;
     }
+
+    pthread_mutex_unlock(&ctx->lock);
 }
 
 bool encode_lavc_didfail(struct encode_lavc_context *ctx)
 {
-    return ctx && ctx->failed;
+    if (!ctx)
+        return false;
+    pthread_mutex_lock(&ctx->lock);
+    bool fail = ctx && ctx->failed;
+    pthread_mutex_unlock(&ctx->lock);
+    return fail;
 }
 
 void encode_lavc_fail(struct encode_lavc_context *ctx, const char *format, ...)
