@@ -583,6 +583,11 @@ static struct demuxer *open_given_type(struct mpv_global *global,
         .filename = talloc_strdup(demuxer, stream->url),
         .metadata = talloc_zero(demuxer, struct mp_tags),
     };
+    demuxer->seekable = stream->seekable;
+    if (demuxer->stream->uncached_stream &&
+        !demuxer->stream->uncached_stream->seekable)
+        demuxer->seekable = false;
+
     demuxer->params = params; // temporary during open()
     int64_t start_pos = stream_tell(stream);
 
