@@ -129,6 +129,9 @@ static const struct format_hack format_hacks[] = {
 
 static const char *const format_blacklist[] = {
     "tty",      // Useless non-sense, sometimes breaks MLP2 subreader.c fallback
+    // Image demuxers, disabled in favor of demux_mf:
+    "image2", "image2pipe",
+    "bmp_pipe", "dpx_pipe", "exr_pipe", "j2k_pipe", "png_pipe", "tiff_pipe",
     0
 };
 
@@ -504,7 +507,7 @@ static void handle_stream(demuxer_t *demuxer, int i)
 #if HAVE_AV_DISPLAYMATRIX
         uint8_t *sd = av_stream_get_side_data(st, AV_PKT_DATA_DISPLAYMATRIX, NULL);
         if (sd)
-            sh_video->rotate = av_display_rotation_get((uint32_t *)sd);
+            sh_video->rotate = -av_display_rotation_get((uint32_t *)sd);
 #else
         AVDictionaryEntry *rot = av_dict_get(st->metadata, "rotate", NULL, 0);
         if (rot && rot->value) {
@@ -1006,11 +1009,12 @@ redo:
          * call the new API instead of relying on av_seek_frame() to do this
          * for us.
          */
-        stream_drop_buffers(demuxer->stream);
-        avio_flush(priv->avfc->pb);
         av_seek_frame(priv->avfc, 0, stream_tell(demuxer->stream),
                       AVSEEK_FLAG_BYTE);
-        avio_flush(priv->avfc->pb);
+        // avio_flush() is designed for write-only streams, and does the wrong
+        // thing when reading. Flush it manually instead.
+        priv->avfc->pb->buf_ptr = priv->avfc->pb->buf_end;
+        stream_drop_buffers(demuxer->stream);
         return DEMUXER_CTRL_OK;
     default:
         return DEMUXER_CTRL_NOTIMPL;
