@@ -67,7 +67,7 @@ struct sh_stream {
 typedef struct sh_audio {
     int samplerate;
     struct mp_chmap channels;
-    int i_bps; // == bitrate  (compressed bytes/sec)
+    int bitrate; // compressed bits/sec
     // win32-compatible codec parameters:
     MP_WAVEFORMATEX *wf;
     // note codec extradata may be either under "wf" or "codecdata"
@@ -79,8 +79,9 @@ typedef struct sh_video {
     bool avi_dts;         // use DTS timing; first frame and DTS is 0
     float fps;            // frames per second (set only if constant fps)
     float aspect;         // aspect ratio stored in the file (for prescaling)
-    int i_bps;            // == bitrate  (compressed bytes/sec)
+    int bitrate;          // compressed bits/sec
     int disp_w, disp_h;   // display size
+    int rotate;           // intended display rotation, in degrees, [0, 359]
     MP_BITMAPINFOHEADER *bih;
 } sh_video_t;
 
@@ -88,9 +89,9 @@ typedef struct sh_sub {
     unsigned char *extradata;   // extra header data passed from demuxer
     int extradata_len;
     int w, h;                   // mp4 vobsubs
-    int frame_based;            // timestamps are frame-based
+    double frame_based;         // timestamps are frame-based (and this is the
+                                // fallback framerate used for timestamps)
     bool is_utf8;               // if false, subtitle packet charset is unknown
-    struct ass_track *track;    // loaded by libass
     struct dec_sub *dec_sub;    // decoder context
 } sh_sub_t;
 
