@@ -21,7 +21,7 @@
 #ifndef MPLAYER_KEYCODES_H
 #define MPLAYER_KEYCODES_H
 
-// Key in the range [0, MP_KEY_BASE) follow unicode.
+// Keys in the range [0, MP_KEY_BASE) follow unicode.
 // Special keys come after this.
 #define MP_KEY_BASE (1<<21)
 
@@ -222,45 +222,45 @@
     (MP_KEY_IS_MOUSE_BTN_SINGLE(code) || MP_KEY_IS_MOUSE_BTN_DBL(code) || \
      (code) == MP_KEY_MOUSE_MOVE)
 
+// Emit a command even on key-up (normally key-up is ignored). This means by
+// default they binding will be triggered on key-up instead of key-down.
+// This is a fixed part of the keycode, not a modifier than can change.
+#define MP_KEY_EMIT_ON_UP      (1<<22)
+
+// Use this when the key shouldn't be auto-repeated (like mouse buttons)
+// Also means both key-down key-up events produce emit bound commands.
+// This is a fixed part of the keycode, not a modifier than can change.
+#define MP_NO_REPEAT_KEY       (1<<23)
+
 /* Modifiers added to individual keys */
-#define MP_KEY_MODIFIER_SHIFT  (1<<22)
-#define MP_KEY_MODIFIER_CTRL   (1<<23)
-#define MP_KEY_MODIFIER_ALT    (1<<24)
-#define MP_KEY_MODIFIER_META   (1<<25)
+#define MP_KEY_MODIFIER_SHIFT  (1<<24)
+#define MP_KEY_MODIFIER_CTRL   (1<<25)
+#define MP_KEY_MODIFIER_ALT    (1<<26)
+#define MP_KEY_MODIFIER_META   (1<<27)
+
+// Flag for key events. Multiple down events are idempotent. Release keys by
+// sending the key code with KEY_STATE_UP set, or by sending
+// MP_INPUT_RELEASE_ALL as key code.
+#define MP_KEY_STATE_DOWN      (1<<28)
+
+// Flag for key events. Releases a key previously held down with
+// MP_KEY_STATE_DOWN. Do not send redundant UP events and do not forget to
+// release keys at all with UP. If input is unreliable, use MP_INPUT_RELEASE_ALL
+// or don't use MP_KEY_STATE_DOWN in the first place.
+#define MP_KEY_STATE_UP        (1<<29)
 
 #define MP_KEY_MODIFIER_MASK (MP_KEY_MODIFIER_SHIFT | MP_KEY_MODIFIER_CTRL | \
                               MP_KEY_MODIFIER_ALT | MP_KEY_MODIFIER_META | \
                               MP_KEY_STATE_DOWN | MP_KEY_STATE_UP)
 
-// Flag for key events. Multiple down events are idempotent. Release keys by
-// sending the key code with KEY_STATE_UP set, or by sending
-// MP_INPUT_RELEASE_ALL as key code.
-#define MP_KEY_STATE_DOWN      (1<<26)
-
-// Flag for key events. Releases a key previously held down with
-// MP_KEY_STATE_DOWN. Do not sending redundant UP events and do not forget to
-// release keys at all with UP. If input is unreliable, use MP_INPUT_RELEASE_ALL
-// or don't use MP_KEY_STATE_DOWN in the first place.
-#define MP_KEY_STATE_UP        (1<<27)
-
-// The following flags are not modifiers, but are part of the keycode itself.
-
-// Emit a command even on key-up (normally key-up is ignored). The command
-// handling code has to ignore unwanted commands specifically.
-#define MP_KEY_EMIT_ON_UP      (1<<28)
-
-// Use this when the key shouldn't be auto-repeated (like mouse buttons)
-// Also means both key-down key-up events produce emit bound commands.
-#define MP_NO_REPEAT_KEY       (1<<29)
-
 // Get input key from its name.
 int mp_input_get_key_from_name(const char *name);
 
-// Append given key by name to ret, return ret.
-char *mp_input_get_key_name(int key, char *ret);
+// Return given key (plus modifiers) as talloc'ed name.
+char *mp_input_get_key_name(int key);
 
 // Combination of multiple keys to string.
-char *mp_input_get_key_combo_name(int *keys, int max);
+char *mp_input_get_key_combo_name(const int *keys, int max);
 
 // String containing combination of multiple string to keys.
 int mp_input_get_keys_from_string(char *str, int max_num_keys,
