@@ -22,6 +22,7 @@
 
 #include "common/msg.h"
 #include "input/input.h"
+#include "input/event.h"
 #include "input/keycodes.h"
 
 #include "osdep/macosx_application_objc.h"
@@ -55,21 +56,6 @@ Application *mpv_shared_app(void)
     return (Application *)[Application sharedApplication];
 }
 
-static NSString *escape_loadfile_name(NSString *input)
-{
-    NSArray *mappings = @[
-        @{ @"in": @"\\", @"out": @"\\\\" },
-        @{ @"in": @"\"", @"out": @"\\\"" },
-    ];
-
-    for (NSDictionary *mapping in mappings) {
-        input = [input stringByReplacingOccurrencesOfString:mapping[@"in"]
-                                                 withString:mapping[@"out"]];
-    }
-
-    return input;
-}
-
 @implementation Application
 @synthesize files = _files;
 @synthesize argumentsList = _arguments_list;
@@ -278,7 +264,8 @@ static NSString *escape_loadfile_name(NSString *input)
         }
     }];
 
-    self.files = [filesToOpen sortedArrayUsingSelector:@selector(compare:)];
+    SEL cmpsel = @selector(localizedStandardCompare:);
+    self.files = [filesToOpen sortedArrayUsingSelector:cmpsel];
     if (self.willStopOnOpenEvent) {
         self.willStopOnOpenEvent = NO;
         cocoa_stop_runloop();
@@ -287,17 +274,23 @@ static NSString *escape_loadfile_name(NSString *input)
     }
 }
 
+
+- (void)handleFilesArray:(NSArray *)files
+{
+    size_t num_files  = [files count];
+    char **files_utf8 = talloc_array(NULL, char*, num_files);
+    [files enumerateObjectsUsingBlock:^(id obj, NSUInteger i, BOOL *_){
+        char *filename = (char *)[obj UTF8String];
+        size_t bytes   = [obj lengthOfBytesUsingEncoding:NSUTF8StringEncoding];
+        files_utf8[i]  = talloc_memdup(files_utf8, filename, bytes + 1);
+    }];
+    mp_event_drop_files(self.inputContext, num_files, files_utf8);
+    talloc_free(files_utf8);
+}
+
 - (void)handleFiles
 {
-    void *ctx = talloc_new(NULL);
-    [self.files enumerateObjectsUsingBlock:^(id obj, NSUInteger i, BOOL *_){
-        const char *file = [escape_loadfile_name(obj) UTF8String];
-        const char *append = (i == 0) ? "" : " append";
-        char *cmd = talloc_asprintf(ctx, "raw loadfile \"%s\"%s", file, append);
-        mp_cmd_t *cmdt = mp_input_parse_cmd(self.inputContext, bstr0(cmd), "");
-        mp_input_queue_cmd(self.inputContext, cmdt);
-    }];
-    talloc_free(ctx);
+    [self handleFilesArray:self.files];
 }
 @end
 
@@ -474,6 +467,21 @@ static bool bundle_started_from_finder(int argc, char **argv)
     }
 }
 
+static const char macosx_icon[] =
+#include "osdep/macosx_icon.inc"
+;
+
+static void set_application_icon(NSApplication *app)
+{
+    NSData *icon_data = [NSData dataWithBytesNoCopy:(void *)macosx_icon
+                                             length:sizeof(macosx_icon)
+                                       freeWhenDone:NO];
+    NSImage *icon = [[NSImage alloc] initWithData:icon_data];
+    [app setApplicationIconImage:icon];
+    [icon release];
+    [icon_data release];
+}
+
 void macosx_finder_args_preinit(int *argc, char ***argv)
 {
     Application *app = mpv_shared_app();
@@ -495,6 +503,7 @@ void macosx_finder_args_preinit(int *argc, char ***argv)
         *argc = cocoa_argc;
         *argv = cocoa_argv;
     } else {
+        set_application_icon(app);
         for (int i = 0; i < *argc; i++ ) {
             NSString *arg = [NSString stringWithUTF8String:(*argv)[i]];
             [app.argumentsList addObject:arg];
