@@ -37,7 +37,6 @@
 #include "bstr/bstr.h"
 #include "common/common.h"
 #include "common/playlist.h"
-#include "common/playlist_parser.h"
 #include "stream/stream.h"
 
 struct find_entry {
@@ -120,7 +119,7 @@ static int enable_cache(struct MPContext *mpctx, struct stream **stream,
 {
     struct MPOpts *opts = mpctx->opts;
 
-    if (opts->stream_cache_size <= 0)
+    if (!stream_wants_cache(*stream, &opts->stream_cache))
         return 0;
 
     char *filename = talloc_strdup(NULL, (*demuxer)->filename);
@@ -133,11 +132,7 @@ static int enable_cache(struct MPContext *mpctx, struct stream **stream,
         return -1;
     }
 
-    stream_enable_cache_percent(stream,
-                                opts->stream_cache_size,
-                                opts->stream_cache_def_size,
-                                opts->stream_cache_min_percent,
-                                opts->stream_cache_seek_min_percent);
+    stream_enable_cache(stream, &opts->stream_cache);
 
     *demuxer = demux_open(*stream, "mkv", params, mpctx->global);
     if (!*demuxer) {
