@@ -443,7 +443,6 @@ static int fill_buffer(stream_t *s, char *buf, int max_len)
                 if (priv->title > 0 && tit != priv->title) {
                     priv->next_event |= 1 << MP_NAV_EVENT_EOF;;
                     MP_WARN(s, "Requested title not found\n");
-                    return 0;
                 }
             }
             if (priv->nav_enabled)
@@ -546,7 +545,11 @@ static int control(stream_t *stream, int cmd, void *arg)
         return STREAM_OK;
     }
     case STREAM_CTRL_SEEK_TO_TIME: {
-        uint64_t tm = (uint64_t) (*((double *)arg) * 90000);
+        int64_t tm = (int64_t) (*((double *)arg) * 90000);
+        if (tm < 0)
+            tm = 0;
+        if (priv->duration && tm >= (priv->duration * 90))
+            tm = priv->duration * 90 - 1;
         MP_VERBOSE(stream, "seek to PTS %"PRId64"\n", tm);
         if (dvdnav_time_search(dvdnav, tm) != DVDNAV_STATUS_OK)
             break;
@@ -634,6 +637,8 @@ static void stream_dvdnav_close(stream_t *s)
     priv->dvdnav = NULL;
     if (priv->dvd_speed)
         dvd_set_speed(s, priv->filename, -1);
+    if (priv->filename)
+        free(priv->filename);
 }
 
 static struct priv *new_dvdnav_stream(stream_t *stream, char *filename)
@@ -692,7 +697,7 @@ static int open_s(stream_t *stream)
         int best_title = -1;
         int32_t num_titles;
         if (dvdnav_get_number_of_titles(dvdnav, &num_titles) == DVDNAV_STATUS_OK) {
-            for (int n = 1; n < num_titles; n++) {
+            for (int n = 1; n <= num_titles; n++) {
                 uint64_t *parts = NULL, duration = 0;
                 dvdnav_describe_title_chapters(dvdnav, n, &parts, &duration);
                 if (parts) {
