@@ -27,7 +27,7 @@
 
 struct mp_log;
 
-typedef struct tv_param_s {
+typedef struct tv_params {
     char *freq;
     char *channel;
     char *chanlist;
@@ -42,7 +42,7 @@ typedef struct tv_param_s {
     int outfmt;
     float fps;
     char **channels;
-    int noaudio;
+    int audio;
     int immediate;
     int audiorate;
     int audio_id;
@@ -68,22 +68,18 @@ typedef struct tv_param_s {
     int scan;
     int scan_threshold;
     float scan_period;
-    /**
-      Terminate stream with video renderer instead of Null renderer
-      Will help if video freezes but audio does not.
-      May not work with -vo directx and -vf crop combination.
-    */
 } tv_param_t;
 
-extern tv_param_t stream_tv_defaults;
+struct tv_stream_params {
+    char *channel;
+    int input;
+};
 
 typedef struct tvi_info_s
 {
     struct tvi_handle_s * (*tvi_init)(struct mp_log *log, tv_param_t* tv_param);
     const char *name;
     const char *short_name;
-    const char *author;
-    const char *comment;
 } tvi_info_t;
 
 
@@ -103,18 +99,22 @@ typedef struct tvi_functions_s
 
 typedef struct tvi_handle_s {
     struct mp_log       *log;
-    const tvi_functions_t	*functions;
-    void		*priv;
-    int 		seq;
-    struct demuxer	*demuxer;
+    const tvi_functions_t       *functions;
+    void                *priv;
+    int                 seq;
+    struct demuxer      *demuxer;
 
     /* specific */
-    int			norm;
-    int			chanlist;
+    int                 norm;
+    int                 chanlist;
     const struct CHANLIST *chanlist_s;
-    int			channel;
+    int                 channel;
     tv_param_t          * tv_param;
     void                * scan;
+
+    struct tv_channels_s *tv_channel_list;
+    struct tv_channels_s *tv_channel_current, *tv_channel_last;
+    char *tv_channel_last_real;
 } tvi_handle_t;
 
 typedef struct tv_channels_s {
@@ -127,89 +127,81 @@ typedef struct tv_channels_s {
     struct tv_channels_s *prev;
 } tv_channels_t;
 
-extern tv_channels_t *tv_channel_list;
-extern tv_channels_t *tv_channel_current, *tv_channel_last;
-extern char *tv_channel_last_real;
-
 typedef struct {
     unsigned int     scan_timer;
     int     channel_num;
     int     new_channels;
 } tv_scan_t;
 
-#define TVI_CONTROL_FALSE		0
-#define TVI_CONTROL_TRUE		1
-#define TVI_CONTROL_NA			-1
-#define TVI_CONTROL_UNKNOWN		-2
+#define TVI_CONTROL_FALSE               0
+#define TVI_CONTROL_TRUE                1
+#define TVI_CONTROL_NA                  -1
+#define TVI_CONTROL_UNKNOWN             -2
 
 /* ======================== CONTROLS =========================== */
 
 /* GENERIC controls */
-#define TVI_CONTROL_IS_AUDIO		0x1
-#define TVI_CONTROL_IS_VIDEO		0x2
-#define TVI_CONTROL_IS_TUNER		0x3
+#define TVI_CONTROL_IS_AUDIO            0x1
+#define TVI_CONTROL_IS_VIDEO            0x2
+#define TVI_CONTROL_IS_TUNER            0x3
 #define TVI_CONTROL_IMMEDIATE           0x4
 
 /* VIDEO controls */
-#define TVI_CONTROL_VID_GET_FPS		0x101
-#define TVI_CONTROL_VID_GET_PLANES	0x102
-#define TVI_CONTROL_VID_GET_BITS	0x103
-#define TVI_CONTROL_VID_CHK_BITS	0x104
-#define TVI_CONTROL_VID_SET_BITS	0x105
-#define TVI_CONTROL_VID_GET_FORMAT	0x106
-#define TVI_CONTROL_VID_CHK_FORMAT	0x107
-#define TVI_CONTROL_VID_SET_FORMAT	0x108
-#define TVI_CONTROL_VID_GET_WIDTH	0x109
-#define TVI_CONTROL_VID_CHK_WIDTH	0x110
-#define TVI_CONTROL_VID_SET_WIDTH	0x111
-#define TVI_CONTROL_VID_GET_HEIGHT	0x112
-#define TVI_CONTROL_VID_CHK_HEIGHT	0x113
-#define TVI_CONTROL_VID_SET_HEIGHT	0x114
-#define TVI_CONTROL_VID_GET_BRIGHTNESS	0x115
-#define TVI_CONTROL_VID_SET_BRIGHTNESS	0x116
-#define TVI_CONTROL_VID_GET_HUE		0x117
-#define TVI_CONTROL_VID_SET_HUE		0x118
-#define TVI_CONTROL_VID_GET_SATURATION	0x119
-#define TVI_CONTROL_VID_SET_SATURATION	0x11a
-#define TVI_CONTROL_VID_GET_CONTRAST	0x11b
-#define TVI_CONTROL_VID_SET_CONTRAST	0x11c
-#define TVI_CONTROL_VID_GET_PICTURE	0x11d
-#define TVI_CONTROL_VID_SET_PICTURE	0x11e
-#define TVI_CONTROL_VID_SET_GAIN	0x11f
-#define TVI_CONTROL_VID_GET_GAIN	0x120
-#define TVI_CONTROL_VID_SET_WIDTH_HEIGHT	0x121
+#define TVI_CONTROL_VID_GET_FPS         0x101
+#define TVI_CONTROL_VID_GET_PLANES      0x102
+#define TVI_CONTROL_VID_GET_BITS        0x103
+#define TVI_CONTROL_VID_CHK_BITS        0x104
+#define TVI_CONTROL_VID_SET_BITS        0x105
+#define TVI_CONTROL_VID_GET_FORMAT      0x106
+#define TVI_CONTROL_VID_CHK_FORMAT      0x107
+#define TVI_CONTROL_VID_SET_FORMAT      0x108
+#define TVI_CONTROL_VID_GET_WIDTH       0x109
+#define TVI_CONTROL_VID_CHK_WIDTH       0x110
+#define TVI_CONTROL_VID_SET_WIDTH       0x111
+#define TVI_CONTROL_VID_GET_HEIGHT      0x112
+#define TVI_CONTROL_VID_CHK_HEIGHT      0x113
+#define TVI_CONTROL_VID_SET_HEIGHT      0x114
+#define TVI_CONTROL_VID_GET_BRIGHTNESS  0x115
+#define TVI_CONTROL_VID_SET_BRIGHTNESS  0x116
+#define TVI_CONTROL_VID_GET_HUE         0x117
+#define TVI_CONTROL_VID_SET_HUE         0x118
+#define TVI_CONTROL_VID_GET_SATURATION  0x119
+#define TVI_CONTROL_VID_SET_SATURATION  0x11a
+#define TVI_CONTROL_VID_GET_CONTRAST    0x11b
+#define TVI_CONTROL_VID_SET_CONTRAST    0x11c
+#define TVI_CONTROL_VID_GET_PICTURE     0x11d
+#define TVI_CONTROL_VID_SET_PICTURE     0x11e
+#define TVI_CONTROL_VID_SET_GAIN        0x11f
+#define TVI_CONTROL_VID_GET_GAIN        0x120
+#define TVI_CONTROL_VID_SET_WIDTH_HEIGHT        0x121
 
 /* TUNER controls */
-#define TVI_CONTROL_TUN_GET_FREQ	0x201
-#define TVI_CONTROL_TUN_SET_FREQ	0x202
-#define TVI_CONTROL_TUN_GET_TUNER	0x203	/* update priv->tuner struct for used input */
-#define TVI_CONTROL_TUN_SET_TUNER	0x204	/* update priv->tuner struct for used input */
-#define TVI_CONTROL_TUN_GET_NORM	0x205
-#define TVI_CONTROL_TUN_SET_NORM	0x206
-#define TVI_CONTROL_TUN_GET_SIGNAL	0x207
+#define TVI_CONTROL_TUN_GET_FREQ        0x201
+#define TVI_CONTROL_TUN_SET_FREQ        0x202
+#define TVI_CONTROL_TUN_GET_TUNER       0x203   /* update priv->tuner struct for used input */
+#define TVI_CONTROL_TUN_SET_TUNER       0x204   /* update priv->tuner struct for used input */
+#define TVI_CONTROL_TUN_GET_NORM        0x205
+#define TVI_CONTROL_TUN_SET_NORM        0x206
+#define TVI_CONTROL_TUN_GET_SIGNAL      0x207
 
 /* AUDIO controls */
-#define TVI_CONTROL_AUD_GET_FORMAT	0x301
-#define TVI_CONTROL_AUD_GET_SAMPLERATE	0x302
-#define TVI_CONTROL_AUD_GET_CHANNELS	0x304
-#define TVI_CONTROL_AUD_SET_SAMPLERATE	0x305
+#define TVI_CONTROL_AUD_GET_FORMAT      0x301
+#define TVI_CONTROL_AUD_GET_SAMPLERATE  0x302
+#define TVI_CONTROL_AUD_GET_CHANNELS    0x304
+#define TVI_CONTROL_AUD_SET_SAMPLERATE  0x305
 
 /* SPECIFIC controls */
-#define TVI_CONTROL_SPC_GET_INPUT	0x401	/* set input channel (tv,s-video,composite..) */
-#define TVI_CONTROL_SPC_SET_INPUT	0x402	/* set input channel (tv,s-video,composite..) */
-#define TVI_CONTROL_SPC_GET_NORMID	0x403	/* get normid from norm name */
+#define TVI_CONTROL_SPC_GET_INPUT       0x401   /* set input channel (tv,s-video,composite..) */
+#define TVI_CONTROL_SPC_SET_INPUT       0x402   /* set input channel (tv,s-video,composite..) */
+#define TVI_CONTROL_SPC_GET_NORMID      0x403   /* get normid from norm name */
 
 int tv_set_color_options(tvi_handle_t *tvh, int opt, int val);
 int tv_get_color_options(tvi_handle_t *tvh, int opt, int* val);
-#define TV_COLOR_BRIGHTNESS	1
-#define TV_COLOR_HUE		2
-#define TV_COLOR_SATURATION	3
-#define TV_COLOR_CONTRAST	4
 
 int tv_step_channel_real(tvi_handle_t *tvh, int direction);
 int tv_step_channel(tvi_handle_t *tvh, int direction);
-#define TV_CHANNEL_LOWER	1
-#define TV_CHANNEL_HIGHER	2
+#define TV_CHANNEL_LOWER        1
+#define TV_CHANNEL_HIGHER       2
 
 int tv_last_channel(tvi_handle_t *tvh);
 
@@ -231,12 +223,12 @@ void tv_start_scan(tvi_handle_t *tvh, int start);
 tvi_handle_t *tv_new_handle(int size, struct mp_log *log, const tvi_functions_t *functions);
 void tv_free_handle(tvi_handle_t *h);
 
-#define TV_NORM_PAL		1
-#define TV_NORM_NTSC		2
-#define TV_NORM_SECAM		3
-#define TV_NORM_PALNC		4
-#define TV_NORM_PALM		5
-#define TV_NORM_PALN		6
-#define TV_NORM_NTSCJP		7
+#define TV_NORM_PAL             1
+#define TV_NORM_NTSC            2
+#define TV_NORM_SECAM           3
+#define TV_NORM_PALNC           4
+#define TV_NORM_PALM            5
+#define TV_NORM_PALN            6
+#define TV_NORM_NTSCJP          7
 
 #endif /* MPLAYER_TV_H */
