@@ -44,7 +44,6 @@
 
 #include "vd.h"
 #include "video/img_format.h"
-#include "video/mp_image_pool.h"
 #include "video/filter/vf.h"
 #include "video/decode/dec_video.h"
 #include "demux/stheader.h"
@@ -62,41 +61,72 @@
 static void init_avctx(struct dec_video *vd, const char *decoder,
                        struct vd_lavc_hwdec *hwdec);
 static void uninit_avctx(struct dec_video *vd);
-static void setup_refcounting_hw(struct AVCodecContext *s);
 
+static int get_buffer2_hwdec(AVCodecContext *avctx, AVFrame *pic, int flags);
 static enum AVPixelFormat get_format_hwdec(struct AVCodecContext *avctx,
                                            const enum AVPixelFormat *pix_fmt);
 
 static void uninit(struct dec_video *vd);
 
-#define OPT_BASE_STRUCT struct MPOpts
+#define OPT_BASE_STRUCT struct vd_lavc_params
 
-const m_option_t lavc_decode_opts_conf[] = {
-    OPT_FLAG_CONSTANTS("fast", lavc_param.fast, 0, 0, CODEC_FLAG2_FAST),
-    OPT_FLAG("show-all", lavc_param.show_all, 0),
-    OPT_STRING("skiploopfilter", lavc_param.skip_loop_filter_str, 0),
-    OPT_STRING("skipidct", lavc_param.skip_idct_str, 0),
-    OPT_STRING("skipframe", lavc_param.skip_frame_str, 0),
-    OPT_INTRANGE("threads", lavc_param.threads, 0, 0, 16),
-    OPT_FLAG_CONSTANTS("bitexact", lavc_param.bitexact, 0, 0, CODEC_FLAG_BITEXACT),
-    OPT_FLAG("check-hw-profile", lavc_param.check_hw_profile, 0),
-    OPT_STRING("o", lavc_param.avopt, 0),
-    {NULL, NULL, 0, 0, 0, 0, NULL}
+struct vd_lavc_params {
+    int fast;
+    int show_all;
+    int skip_loop_filter;
+    int skip_idct;
+    int skip_frame;
+    int threads;
+    int bitexact;
+    int check_hw_profile;
+    char *avopt;
+};
+
+static const struct m_opt_choice_alternatives discard_names[] = {
+    {"none",        AVDISCARD_NONE},
+    {"default",     AVDISCARD_DEFAULT},
+    {"nonref",      AVDISCARD_NONREF},
+    {"bidir",       AVDISCARD_BIDIR},
+    {"nonkey",      AVDISCARD_NONKEY},
+    {"all",         AVDISCARD_ALL},
+    {0}
+};
+#define OPT_DISCARD(name, field, flags) \
+    OPT_GENERAL(int, name, field, flags, .type = CONF_TYPE_CHOICE, \
+                .priv = (void *)discard_names)
+
+const struct m_sub_options vd_lavc_conf = {
+    .opts = (const m_option_t[]){
+        OPT_FLAG("fast", fast, 0),
+        OPT_FLAG("show-all", show_all, 0),
+        OPT_DISCARD("skiploopfilter", skip_loop_filter, 0),
+        OPT_DISCARD("skipidct", skip_idct, 0),
+        OPT_DISCARD("skipframe", skip_frame, 0),
+        OPT_INTRANGE("threads", threads, 0, 0, 16),
+        OPT_FLAG("bitexact", bitexact, 0),
+        OPT_FLAG("check-hw-profile", check_hw_profile, 0),
+        OPT_STRING("o", avopt, 0),
+        {0}
+    },
+    .size = sizeof(struct vd_lavc_params),
+    .defaults = &(const struct vd_lavc_params){
+        .show_all = 0,
+        .check_hw_profile = 1,
+        .skip_loop_filter = AVDISCARD_DEFAULT,
+        .skip_idct = AVDISCARD_DEFAULT,
+        .skip_frame = AVDISCARD_DEFAULT,
+    },
 };
 
 const struct vd_lavc_hwdec mp_vd_lavc_vdpau;
-const struct vd_lavc_hwdec mp_vd_lavc_vdpau_old;
 const struct vd_lavc_hwdec mp_vd_lavc_vda;
 const struct vd_lavc_hwdec mp_vd_lavc_vaapi;
 const struct vd_lavc_hwdec mp_vd_lavc_vaapi_copy;
 
-static const struct vd_lavc_hwdec *hwdec_list[] = {
+static const struct vd_lavc_hwdec *const hwdec_list[] = {
 #if HAVE_VDPAU_HWACCEL
     &mp_vd_lavc_vdpau,
 #endif
-#if HAVE_VDPAU_DECODER
-    &mp_vd_lavc_vdpau_old,
-#endif
 #if HAVE_VDA_HWACCEL
     &mp_vd_lavc_vda,
 #endif
@@ -128,26 +158,13 @@ static bool hwdec_codec_allowed(struct dec_video *vd, const char *codec)
     return false;
 }
 
-static enum AVDiscard str2AVDiscard(struct dec_video *vd, char *str)
-{
-    if (!str)                               return AVDISCARD_DEFAULT;
-    if (strcasecmp(str, "none"   ) == 0)    return AVDISCARD_NONE;
-    if (strcasecmp(str, "default") == 0)    return AVDISCARD_DEFAULT;
-    if (strcasecmp(str, "nonref" ) == 0)    return AVDISCARD_NONREF;
-    if (strcasecmp(str, "bidir"  ) == 0)    return AVDISCARD_BIDIR;
-    if (strcasecmp(str, "nonkey" ) == 0)    return AVDISCARD_NONKEY;
-    if (strcasecmp(str, "all"    ) == 0)    return AVDISCARD_ALL;
-    MP_ERR(vd, "Unknown discard value %s\n", str);
-    return AVDISCARD_DEFAULT;
-}
-
 // Find the correct profile entry for the current codec and profile.
 // Assumes the table has higher profiles first (for each codec).
 const struct hwdec_profile_entry *hwdec_find_profile(
     struct lavc_ctx *ctx, const struct hwdec_profile_entry *table)
 {
     assert(AV_CODEC_ID_NONE == 0);
-    struct lavc_param *lavc_param = &ctx->opts->lavc_param;
+    struct vd_lavc_params *lavc_param = ctx->opts->vd_lavc_params;
     enum AVCodecID codec = ctx->avctx->codec_id;
     int profile = ctx->avctx->profile;
     // Assume nobody cares about these aspects of the profile
@@ -191,52 +208,40 @@ void hwdec_request_api(struct mp_hwdec_info *info, const char *api_name)
 }
 
 static int hwdec_probe(struct vd_lavc_hwdec *hwdec, struct mp_hwdec_info *info,
-                       const char *decoder, const char **hw_decoder)
+                       const char *decoder)
 {
-    if (hwdec->codec_pairs) {
-        for (int n = 0; hwdec->codec_pairs[n + 0]; n += 2) {
-            const char *sw = hwdec->codec_pairs[n + 0];
-            const char *hw = hwdec->codec_pairs[n + 1];
-            if (decoder && strcmp(decoder, sw) == 0) {
-                AVCodec *codec = avcodec_find_decoder_by_name(hw);
-                *hw_decoder = hw;
-                if (codec)
-                    goto found;
-            }
-        }
-        return HWDEC_ERR_NO_CODEC;
-    found: ;
-    }
     int r = 0;
     if (hwdec->probe)
         r = hwdec->probe(hwdec, info, decoder);
     return r;
 }
 
-static bool probe_hwdec(struct dec_video *vd, bool autoprobe, enum hwdec_type api,
-                        const char *decoder, struct vd_lavc_hwdec **use_hwdec,
-                        const char **use_decoder)
+static struct vd_lavc_hwdec *probe_hwdec(struct dec_video *vd, bool autoprobe,
+                                         enum hwdec_type api,
+                                         const char *decoder)
 {
     struct vd_lavc_hwdec *hwdec = find_hwcodec(api);
     if (!hwdec) {
-        MP_VERBOSE(vd, "Requested hardware decoder not "
-                   "compiled.\n");
-        return false;
+        MP_VERBOSE(vd, "Requested hardware decoder not compiled.\n");
+        return NULL;
+    }
+    int r = hwdec_probe(hwdec, &vd->hwdec_info, decoder);
+    if (r == HWDEC_ERR_EMULATED) {
+        if (autoprobe)
+            return NULL;
+        // User requested this explicitly.
+        MP_WARN(vd, "Using emulated hardware decoding API.\n");
+        r = 0;
     }
-    const char *hw_decoder = NULL;
-    int r = hwdec_probe(hwdec, &vd->hwdec_info, decoder, &hw_decoder);
     if (r >= 0) {
-        *use_hwdec = hwdec;
-        *use_decoder = hw_decoder;
-        return true;
+        return hwdec;
     } else if (r == HWDEC_ERR_NO_CODEC) {
         MP_VERBOSE(vd, "Hardware decoder '%s' not found in "
-                   "libavcodec.\n", hw_decoder ? hw_decoder : decoder);
+                   "libavcodec.\n", decoder);
     } else if (r == HWDEC_ERR_NO_CTX && !autoprobe) {
-        MP_WARN(vd, "VO does not support requested "
-                "hardware decoder.\n");
+        MP_WARN(vd, "VO does not support requested hardware decoder.\n");
     }
-    return false;
+    return NULL;
 }
 
 
@@ -246,7 +251,8 @@ static int init(struct dec_video *vd, const char *decoder)
     ctx = vd->priv = talloc_zero(NULL, vd_ffmpeg_ctx);
     ctx->log = vd->log;
     ctx->opts = vd->opts;
-    ctx->non_dr1_pool = talloc_steal(ctx, mp_image_pool_new(16));
+
+    ctx->selected_hwdec = vd->opts->hwdec_api;
 
     if (bstr_endswith0(bstr0(decoder), "_vdpau")) {
         MP_WARN(vd, "VDPAU decoder '%s' was requested. "
@@ -260,18 +266,16 @@ static int init(struct dec_video *vd, const char *decoder)
     }
 
     struct vd_lavc_hwdec *hwdec = NULL;
-    const char *hw_decoder = NULL;
 
     if (hwdec_codec_allowed(vd, decoder)) {
         if (vd->opts->hwdec_api == HWDEC_AUTO) {
             for (int n = 0; hwdec_list[n]; n++) {
-                if (probe_hwdec(vd, true, hwdec_list[n]->type, decoder,
-                    &hwdec, &hw_decoder))
+                hwdec = probe_hwdec(vd, true, hwdec_list[n]->type, decoder);
+                if (hwdec)
                     break;
             }
         } else if (vd->opts->hwdec_api != HWDEC_NONE) {
-            probe_hwdec(vd, false, vd->opts->hwdec_api, decoder,
-                        &hwdec, &hw_decoder);
+            hwdec = probe_hwdec(vd, false, vd->opts->hwdec_api, decoder);
         }
     } else {
         MP_VERBOSE(vd, "Not trying to use hardware decoding: "
@@ -280,8 +284,6 @@ static int init(struct dec_video *vd, const char *decoder)
 
     if (hwdec) {
         ctx->software_fallback_decoder = talloc_strdup(ctx, decoder);
-        if (hw_decoder)
-            decoder = hw_decoder;
         MP_INFO(vd, "Trying to use hardware decoding.\n");
     } else if (vd->opts->hwdec_api != HWDEC_NONE) {
         MP_INFO(vd, "Using software decoding.\n");
@@ -301,50 +303,18 @@ static int init(struct dec_video *vd, const char *decoder)
             return 0;
         }
     }
+
+    if (ctx->avctx->bit_rate != 0)
+        vd->bitrate = ctx->avctx->bit_rate;
+
     return 1;
 }
 
 static void set_from_bih(AVCodecContext *avctx, uint32_t format,
                          MP_BITMAPINFOHEADER *bih)
 {
-
-    switch (format) {
-    case MP_FOURCC('S','V','Q','3'):
-    case MP_FOURCC('A','V','R','n'):
-    case MP_FOURCC('M','J','P','G'):
-        /* AVRn stores huffman table in AVI header */
-        /* Pegasus MJPEG stores it also in AVI header, but it uses the common
-         * MJPG fourcc :( */
-        if (bih->biSize <= sizeof(*bih))
-           break;
-        av_opt_set_int(avctx, "extern_huff", 1, AV_OPT_SEARCH_CHILDREN);
+    if (bih->biSize > sizeof(*bih))
         mp_lavc_set_extradata(avctx, bih + 1, bih->biSize - sizeof(*bih));
-        break;
-
-    case MP_FOURCC('R','V','1','0'):
-    case MP_FOURCC('R','V','1','3'):
-    case MP_FOURCC('R','V','2','0'):
-    case MP_FOURCC('R','V','3','0'):
-    case MP_FOURCC('R','V','4','0'):
-        if (bih->biSize < sizeof(*bih) + 8) {
-            // only 1 packet per frame & sub_id from fourcc
-            uint32_t extradata[2] = {
-                0,
-                format == MP_FOURCC('R','V','1','3') ? 0x10003001 : 0x10000000,
-            };
-            mp_lavc_set_extradata(avctx, &extradata, 8);
-        } else {
-            // has extra slice header (demux_rm or rm->avi streamcopy)
-            mp_lavc_set_extradata(avctx, bih + 1, bih->biSize - sizeof(*bih));
-        }
-        break;
-
-    default:
-        if (bih->biSize <= sizeof(*bih))
-            break;
-        mp_lavc_set_extradata(avctx, bih + 1, bih->biSize - sizeof(*bih));
-        break;
-    }
 
     avctx->bits_per_coded_sample = bih->biBitCount;
     avctx->coded_width  = bih->biWidth;
@@ -355,7 +325,7 @@ static void init_avctx(struct dec_video *vd, const char *decoder,
                        struct vd_lavc_hwdec *hwdec)
 {
     vd_ffmpeg_ctx *ctx = vd->priv;
-    struct lavc_param *lavc_param = &vd->opts->lavc_param;
+    struct vd_lavc_params *lavc_param = vd->opts->vd_lavc_params;
     bool mp_rawvideo = false;
     struct sh_stream *sh = vd->header;
 
@@ -372,45 +342,34 @@ static void init_avctx(struct dec_video *vd, const char *decoder,
 
     ctx->hwdec_info = &vd->hwdec_info;
 
-    ctx->do_dr1 = ctx->do_hw_dr1 = 0;
     ctx->pix_fmt = AV_PIX_FMT_NONE;
     ctx->hwdec = hwdec;
+    ctx->hwdec_fmt = 0;
     ctx->avctx = avcodec_alloc_context3(lavc_codec);
     AVCodecContext *avctx = ctx->avctx;
+    avctx->bit_rate = 0;
     avctx->opaque = vd;
     avctx->codec_type = AVMEDIA_TYPE_VIDEO;
     avctx->codec_id = lavc_codec->id;
 
-#if HAVE_AVUTIL_REFCOUNTING
     avctx->refcounted_frames = 1;
     ctx->pic = av_frame_alloc();
-#else
-    ctx->pic = avcodec_alloc_frame();
-#endif
 
     if (ctx->hwdec) {
-        ctx->do_hw_dr1         = true;
         avctx->thread_count    = 1;
         avctx->get_format      = get_format_hwdec;
-        setup_refcounting_hw(avctx);
+        avctx->get_buffer2     = get_buffer2_hwdec;
         if (ctx->hwdec->init(ctx) < 0) {
             uninit_avctx(vd);
             return;
         }
     } else {
-#if !HAVE_AVUTIL_REFCOUNTING
-        if (lavc_codec->capabilities & CODEC_CAP_DR1) {
-            ctx->do_dr1            = true;
-            avctx->get_buffer      = mp_codec_get_buffer;
-            avctx->release_buffer  = mp_codec_release_buffer;
-        }
-#endif
         mp_set_avcodec_threads(avctx, lavc_param->threads);
     }
 
-    avctx->flags |= lavc_param->bitexact;
+    avctx->flags |= lavc_param->bitexact ? CODEC_FLAG_BITEXACT : 0;
+    avctx->flags2 |= lavc_param->fast ? CODEC_FLAG2_FAST : 0;
 
-    avctx->flags2 |= lavc_param->fast;
     if (lavc_param->show_all) {
 #ifdef CODEC_FLAG2_SHOW_ALL
         avctx->flags2 |= CODEC_FLAG2_SHOW_ALL; // ffmpeg only?
@@ -420,9 +379,9 @@ static void init_avctx(struct dec_video *vd, const char *decoder,
 #endif
     }
 
-    avctx->skip_loop_filter = str2AVDiscard(vd, lavc_param->skip_loop_filter_str);
-    avctx->skip_idct = str2AVDiscard(vd, lavc_param->skip_idct_str);
-    avctx->skip_frame = str2AVDiscard(vd, lavc_param->skip_frame_str);
+    avctx->skip_loop_filter = lavc_param->skip_loop_filter;
+    avctx->skip_idct = lavc_param->skip_idct;
+    avctx->skip_frame = lavc_param->skip_frame;
 
     if (lavc_param->avopt) {
         if (parse_avopts(avctx, lavc_param->avopt) < 0) {
@@ -468,6 +427,9 @@ static void uninit_avctx(struct dec_video *vd)
     vd_ffmpeg_ctx *ctx = vd->priv;
     AVCodecContext *avctx = ctx->avctx;
 
+    if (ctx->hwdec && ctx->hwdec->uninit)
+        ctx->hwdec->uninit(ctx);
+
     if (avctx) {
         if (avctx->codec && avcodec_close(avctx) < 0)
             MP_ERR(vd, "Could not close codec.\n");
@@ -478,15 +440,7 @@ static void uninit_avctx(struct dec_video *vd)
 
     av_freep(&ctx->avctx);
 
-    if (ctx->hwdec && ctx->hwdec->uninit)
-        ctx->hwdec->uninit(ctx);
-
-#if HAVE_AVUTIL_REFCOUNTING
     av_frame_free(&ctx->pic);
-#else
-    avcodec_free_frame(&ctx->pic);
-    mp_buffer_pool_free(&ctx->dr1_buffer_pool);
-#endif
 }
 
 static void uninit(struct dec_video *vd)
@@ -498,15 +452,12 @@ static void update_image_params(struct dec_video *vd, AVFrame *frame,
                                 struct mp_image_params *out_params)
 {
     vd_ffmpeg_ctx *ctx = vd->priv;
+    struct MPOpts *opts = ctx->opts;
     int width = frame->width;
     int height = frame->height;
     float aspect = av_q2d(frame->sample_aspect_ratio) * width / height;
     int pix_fmt = frame->format;
 
-#if LIBAVCODEC_VERSION_INT < AV_VERSION_INT(54, 40, 0)
-    pix_fmt = ctx->avctx->pix_fmt;
-#endif
-
     if (pix_fmt != ctx->pix_fmt) {
         ctx->pix_fmt = pix_fmt;
         ctx->best_csp = pixfmt2imgfmt(pix_fmt);
@@ -526,9 +477,17 @@ static void update_image_params(struct dec_video *vd, AVFrame *frame,
         .d_h = d_h,
         .colorspace = avcol_spc_to_mp_csp(ctx->avctx->colorspace),
         .colorlevels = avcol_range_to_mp_csp_levels(ctx->avctx->color_range),
+        .primaries = avcol_pri_to_mp_csp_prim(ctx->avctx->color_primaries),
         .chroma_location =
             avchroma_location_to_mp(ctx->avctx->chroma_sample_location),
+        .rotate = vd->header->video->rotate,
     };
+
+    if (opts->video_rotate < 0) {
+        out_params->rotate = 0;
+    } else {
+        out_params->rotate = (out_params->rotate + opts->video_rotate) % 360;
+    }
 }
 
 static enum AVPixelFormat get_format_hwdec(struct AVCodecContext *avctx,
@@ -544,11 +503,30 @@ static enum AVPixelFormat get_format_hwdec(struct AVCodecContext *avctx,
 
     assert(ctx->hwdec);
 
-    for (int i = 0; fmt[i] != AV_PIX_FMT_NONE; i++) {
-        const int *okfmt = ctx->hwdec->image_formats;
-        for (int n = 0; okfmt && okfmt[n]; n++) {
-            if (imgfmt2pixfmt(okfmt[n]) == fmt[i])
+    if (ctx->hwdec->image_format) {
+        for (int i = 0; fmt[i] != AV_PIX_FMT_NONE; i++) {
+            if (ctx->hwdec->image_format == pixfmt2imgfmt(fmt[i])) {
+                // There could be more reasons for a change, and it's possible
+                // that we miss some. (Might also depend on the hwaccel type.)
+                bool change =
+                    ctx->hwdec_w != avctx->width ||
+                    ctx->hwdec_h != avctx->height ||
+                    ctx->hwdec_fmt != ctx->hwdec->image_format ||
+                    ctx->hwdec_profile != avctx->profile;
+                ctx->hwdec_w = avctx->width;
+                ctx->hwdec_h = avctx->height;
+                ctx->hwdec_fmt = ctx->hwdec->image_format;
+                ctx->hwdec_profile = avctx->profile;
+                if (ctx->hwdec->init_decoder && change) {
+                    if (ctx->hwdec->init_decoder(ctx, ctx->hwdec_fmt,
+                                                 ctx->hwdec_w, ctx->hwdec_h) < 0)
+                    {
+                        ctx->hwdec_fmt = 0;
+                        break;
+                    }
+                }
                 return fmt[i];
+            }
         }
     }
 
@@ -571,7 +549,7 @@ static struct mp_image *get_surface_hwdec(struct dec_video *vd, AVFrame *pic)
      * get_buffer callback.
      */
     int imgfmt = pixfmt2imgfmt(pic->format);
-    if (!IMGFMT_IS_HWACCEL(imgfmt))
+    if (!IMGFMT_IS_HWACCEL(imgfmt) || !ctx->hwdec)
         return NULL;
 
     // Using frame->width/height is bad. For non-mod 16 video (which would
@@ -581,6 +559,11 @@ static struct mp_image *get_surface_hwdec(struct dec_video *vd, AVFrame *pic)
     int w = ctx->avctx->width;
     int h = ctx->avctx->height;
 
+    if (ctx->hwdec->init_decoder) {
+        if (imgfmt != ctx->hwdec_fmt && w != ctx->hwdec_w && h != ctx->hwdec_h)
+            return NULL;
+    }
+
     struct mp_image *mpi = ctx->hwdec->allocate_image(ctx, imgfmt, w, h);
 
     if (mpi) {
@@ -591,8 +574,6 @@ static struct mp_image *get_surface_hwdec(struct dec_video *vd, AVFrame *pic)
     return mpi;
 }
 
-#if HAVE_AVUTIL_REFCOUNTING
-
 static void free_mpi(void *opaque, uint8_t *data)
 {
     struct mp_image *mpi = opaque;
@@ -612,120 +593,6 @@ static int get_buffer2_hwdec(AVCodecContext *avctx, AVFrame *pic, int flags)
     return 0;
 }
 
-static void setup_refcounting_hw(AVCodecContext *avctx)
-{
-    avctx->get_buffer2 = get_buffer2_hwdec;
-}
-
-#else /* HAVE_AVUTIL_REFCOUNTING */
-
-static int get_buffer_hwdec(AVCodecContext *avctx, AVFrame *pic)
-{
-    struct dec_video *vd = avctx->opaque;
-
-    struct mp_image *mpi = get_surface_hwdec(vd, pic);
-    if (!mpi)
-        return -1;
-
-    pic->opaque = mpi;
-    pic->type = FF_BUFFER_TYPE_USER;
-
-    /* The libavcodec reordered_opaque functionality is implemented by
-     * a similar copy in avcodec_default_get_buffer() and without a
-     * workaround like this it'd stop working when a custom buffer
-     * callback is used.
-     */
-    pic->reordered_opaque = avctx->reordered_opaque;
-    return 0;
-}
-
-static void release_buffer_hwdec(AVCodecContext *avctx, AVFrame *pic)
-{
-    mp_image_t *mpi = pic->opaque;
-
-    assert(pic->type == FF_BUFFER_TYPE_USER);
-    assert(mpi);
-
-    talloc_free(mpi);
-
-    for (int i = 0; i < 4; i++)
-        pic->data[i] = NULL;
-}
-
-static void setup_refcounting_hw(AVCodecContext *avctx)
-{
-    avctx->get_buffer = get_buffer_hwdec;
-    avctx->release_buffer = release_buffer_hwdec;
-}
-
-#endif /* HAVE_AVUTIL_REFCOUNTING */
-
-#if HAVE_AVUTIL_REFCOUNTING
-
-static struct mp_image *image_from_decoder(struct dec_video *vd)
-{
-    vd_ffmpeg_ctx *ctx = vd->priv;
-    AVFrame *pic = ctx->pic;
-
-    struct mp_image *img = mp_image_from_av_frame(pic);
-    av_frame_unref(pic);
-
-    return img;
-}
-
-#else /* HAVE_AVUTIL_REFCOUNTING */
-
-static void fb_ref(void *b)
-{
-    mp_buffer_ref(b);
-}
-
-static void fb_unref(void *b)
-{
-    mp_buffer_unref(b);
-}
-
-static bool fb_is_unique(void *b)
-{
-    return mp_buffer_is_unique(b);
-}
-
-static struct mp_image *image_from_decoder(struct dec_video *vd)
-{
-    vd_ffmpeg_ctx *ctx = vd->priv;
-    AVFrame *pic = ctx->pic;
-
-    struct mp_image new = {0};
-    mp_image_copy_fields_from_av_frame(&new, pic);
-
-    struct mp_image *mpi;
-    if (ctx->do_hw_dr1 && pic->opaque) {
-        mpi = pic->opaque; // reordered frame
-        assert(mpi);
-        mpi = mp_image_new_ref(mpi);
-        mp_image_copy_attributes(mpi, &new);
-    } else if (ctx->do_dr1 && pic->opaque) {
-        struct FrameBuffer *fb = pic->opaque;
-        // initial reference for mpi
-        if (!new.planes[0] || !mp_buffer_check(fb)) {
-            // Decoder returned an unreferenced buffer! Taking this would just
-            // lead to an eventual double-free. Nothing we can do about this.
-            // So just say "fuck you" in a nice way.
-            MP_FATAL(vd,
-    "Impossible condition detected! This version of Libav/FFmpeg is not\n"
-    "supported anymore. Please update.\n");
-            return NULL;
-        }
-        mpi = mp_image_new_external_ref(&new, fb, fb_ref, fb_unref,
-                                        fb_is_unique, NULL);
-    } else {
-        mpi = mp_image_pool_new_copy(ctx->non_dr1_pool, &new);
-    }
-    return mpi;
-}
-
-#endif /* HAVE_AVUTIL_REFCOUNTING */
-
 static int decode(struct dec_video *vd, struct demux_packet *packet,
                   int flags, struct mp_image **out_image)
 {
@@ -759,11 +626,11 @@ static int decode(struct dec_video *vd, struct demux_packet *packet,
     vd->codec_pts = mp_pts_from_av(ctx->pic->pkt_pts, NULL);
     vd->codec_dts = mp_pts_from_av(ctx->pic->pkt_dts, NULL);
 
-    // Note: potentially resets ctx->pic as it is transferred to mpi
-    struct mp_image *mpi = image_from_decoder(vd);
+    struct mp_image *mpi = mp_image_from_av_frame(ctx->pic);
+    av_frame_unref(ctx->pic);
     if (!mpi)
-        return 0;
-    assert(mpi->planes[0]);
+        return 0; // mpi==NULL, or OOM
+    assert(mpi->planes[0] || mpi->planes[3]);
     mp_image_set_params(mpi, &params);
 
     if (ctx->hwdec && ctx->hwdec->process_image)
@@ -821,6 +688,13 @@ static int control(struct dec_video *vd, int cmd, void *arg)
             delay += avctx->thread_count - 1;
         *(int *)arg = delay;
         return CONTROL_TRUE;
+    case VDCTRL_GET_HWDEC: {
+        int hwdec = ctx->selected_hwdec;
+        if (!ctx->software_fallback_decoder)
+            hwdec = 0;
+        *(int *)arg = hwdec;
+        return CONTROL_TRUE;
+    }
     case VDCTRL_FORCE_HWDEC_FALLBACK:
         return force_fallback(vd);
     }
