@@ -74,7 +74,7 @@ static int config(struct vf_instance *vf,
     };
 
     return vf_next_config(vf, vf->priv->outw, vf->priv->outh, d_width,
-			  d_height, flags, outfmt);
+                          d_height, flags, outfmt);
 }
 
 static void prepare_image(struct vf_instance *vf, struct mp_image *dmpi,
@@ -96,6 +96,8 @@ static struct mp_image *filter(struct vf_instance *vf, struct mp_image *mpi)
 
     if (vf->priv->opt_top_margin || vf->priv->opt_bottom_margin) {
         struct mp_image *dmpi = vf_alloc_out_image(vf);
+        if (!dmpi)
+            return NULL;
         mp_image_copy_attributes(dmpi, mpi);
         prepare_image(vf, dmpi, mpi);
         talloc_free(mpi);
