@@ -19,9 +19,9 @@
 #include <libavutil/pixdesc.h>
 #include <libavutil/avutil.h>
 
-#include "config.h"
 #include "video/img_format.h"
 #include "fmt-conversion.h"
+#include "config.h"
 
 static const struct {
     int fmt;
@@ -30,31 +30,31 @@ static const struct {
     {IMGFMT_ARGB, AV_PIX_FMT_ARGB},
     {IMGFMT_BGRA, AV_PIX_FMT_BGRA},
     {IMGFMT_BGR24, AV_PIX_FMT_BGR24},
-    {IMGFMT_BGR16_BE, AV_PIX_FMT_RGB565BE},
-    {IMGFMT_BGR16_LE, AV_PIX_FMT_RGB565LE},
-    {IMGFMT_BGR15_BE, AV_PIX_FMT_RGB555BE},
-    {IMGFMT_BGR15_LE, AV_PIX_FMT_RGB555LE},
-    {IMGFMT_BGR12_BE, AV_PIX_FMT_RGB444BE},
-    {IMGFMT_BGR12_LE, AV_PIX_FMT_RGB444LE},
-    {IMGFMT_BGR8,  AV_PIX_FMT_RGB8},
-    {IMGFMT_BGR4,  AV_PIX_FMT_RGB4},
+    {IMGFMT_RGB565_BE, AV_PIX_FMT_RGB565BE},
+    {IMGFMT_RGB565_LE, AV_PIX_FMT_RGB565LE},
+    {IMGFMT_RGB555_BE, AV_PIX_FMT_RGB555BE},
+    {IMGFMT_RGB555_LE, AV_PIX_FMT_RGB555LE},
+    {IMGFMT_RGB444_BE, AV_PIX_FMT_RGB444BE},
+    {IMGFMT_RGB444_LE, AV_PIX_FMT_RGB444LE},
+    {IMGFMT_RGB8,  AV_PIX_FMT_RGB8},
+    {IMGFMT_RGB4,  AV_PIX_FMT_RGB4},
     {IMGFMT_MONO,  AV_PIX_FMT_MONOBLACK},
     {IMGFMT_MONO_W,  AV_PIX_FMT_MONOWHITE},
-    {IMGFMT_RGB4_BYTE,  AV_PIX_FMT_BGR4_BYTE},
-    {IMGFMT_BGR4_BYTE,  AV_PIX_FMT_RGB4_BYTE},
+    {IMGFMT_RGB4_BYTE,  AV_PIX_FMT_RGB4_BYTE},
+    {IMGFMT_BGR4_BYTE,  AV_PIX_FMT_BGR4_BYTE},
     {IMGFMT_RGB48_LE, AV_PIX_FMT_RGB48LE},
     {IMGFMT_RGB48_BE, AV_PIX_FMT_RGB48BE},
     {IMGFMT_ABGR, AV_PIX_FMT_ABGR},
     {IMGFMT_RGBA, AV_PIX_FMT_RGBA},
     {IMGFMT_RGB24, AV_PIX_FMT_RGB24},
-    {IMGFMT_RGB16_BE, AV_PIX_FMT_BGR565BE},
-    {IMGFMT_RGB16_LE, AV_PIX_FMT_BGR565LE},
-    {IMGFMT_RGB15_BE, AV_PIX_FMT_BGR555BE},
-    {IMGFMT_RGB15_LE, AV_PIX_FMT_BGR555LE},
-    {IMGFMT_RGB12_BE, AV_PIX_FMT_BGR444BE},
-    {IMGFMT_RGB12_LE, AV_PIX_FMT_BGR444LE},
-    {IMGFMT_RGB8,  AV_PIX_FMT_BGR8},
-    {IMGFMT_RGB4,  AV_PIX_FMT_BGR4},
+    {IMGFMT_BGR565_BE, AV_PIX_FMT_BGR565BE},
+    {IMGFMT_BGR565_LE, AV_PIX_FMT_BGR565LE},
+    {IMGFMT_BGR555_BE, AV_PIX_FMT_BGR555BE},
+    {IMGFMT_BGR555_LE, AV_PIX_FMT_BGR555LE},
+    {IMGFMT_BGR444_BE, AV_PIX_FMT_BGR444BE},
+    {IMGFMT_BGR444_LE, AV_PIX_FMT_BGR444LE},
+    {IMGFMT_BGR8,  AV_PIX_FMT_BGR8},
+    {IMGFMT_BGR4,  AV_PIX_FMT_BGR4},
     {IMGFMT_PAL8,  AV_PIX_FMT_PAL8},
     {IMGFMT_GBRP,  AV_PIX_FMT_GBRP},
     {IMGFMT_GBRP9_BE,  AV_PIX_FMT_GBRP9BE},
@@ -68,8 +68,7 @@ static const struct {
     {IMGFMT_NV12,  AV_PIX_FMT_NV12},
     {IMGFMT_NV21,  AV_PIX_FMT_NV21},
     {IMGFMT_Y8,    AV_PIX_FMT_GRAY8},
-    // Support really ancient ffmpeg versions (before e91946ed23dfbb)
-    // Newer versions use AV_PIX_FMT_GRAY8A
+    // FFmpeg prefers AV_PIX_FMT_GRAY8A, but Libav has only Y400A
     {IMGFMT_YA8,   AV_PIX_FMT_Y400A},
     {IMGFMT_Y16_LE, AV_PIX_FMT_GRAY16LE},
     {IMGFMT_Y16_BE, AV_PIX_FMT_GRAY16BE},
@@ -108,7 +107,6 @@ static const struct {
 
     {IMGFMT_420AP, AV_PIX_FMT_YUVA420P},
 
-#if LIBAVUTIL_VERSION_MICRO >= 100 && LIBAVUTIL_VERSION_INT >= AV_VERSION_INT(52, 10, 0)
     {IMGFMT_422AP, AV_PIX_FMT_YUVA422P},
     {IMGFMT_444AP, AV_PIX_FMT_YUVA444P},
 
@@ -132,13 +130,9 @@ static const struct {
     {IMGFMT_444AP10_LE, AV_PIX_FMT_YUVA444P10LE},
     {IMGFMT_444AP16_BE, AV_PIX_FMT_YUVA444P16BE},
     {IMGFMT_444AP16_LE, AV_PIX_FMT_YUVA444P16LE},
-#endif
 
-#if LIBAVUTIL_VERSION_INT >= AV_VERSION_INT(52, 25, 0)
-    // Libav added this already at 52.10.0
     {IMGFMT_XYZ12_LE,   AV_PIX_FMT_XYZ12LE},
     {IMGFMT_XYZ12_BE,   AV_PIX_FMT_XYZ12BE},
-#endif
 
     // ffmpeg only
 #if LIBAVUTIL_VERSION_MICRO >= 100
@@ -178,20 +172,12 @@ static const struct {
     {IMGFMT_BGR0,  AV_PIX_FMT_BGRA},
 #endif
 
-#if HAVE_AVCODEC_NEW_VDPAU_API
     {IMGFMT_VDPAU,           AV_PIX_FMT_VDPAU},
+#if HAVE_VDA_AV_VDA_ALLOC_CONTEXT
+    {IMGFMT_VDA,             AV_PIX_FMT_VDA},
 #else
-    {IMGFMT_VDPAU_MPEG1,     AV_PIX_FMT_VDPAU_MPEG1},
-    {IMGFMT_VDPAU_MPEG2,     AV_PIX_FMT_VDPAU_MPEG2},
-    {IMGFMT_VDPAU_H264,      AV_PIX_FMT_VDPAU_H264},
-    {IMGFMT_VDPAU_WMV3,      AV_PIX_FMT_VDPAU_WMV3},
-    {IMGFMT_VDPAU_VC1,       AV_PIX_FMT_VDPAU_VC1},
-    {IMGFMT_VDPAU_MPEG4,     AV_PIX_FMT_VDPAU_MPEG4},
-    // map to an arbitrary but existing vdpau format
-    {IMGFMT_VDPAU,           AV_PIX_FMT_VDPAU_H264},
-#endif
-
     {IMGFMT_VDA,             AV_PIX_FMT_VDA_VLD},
+#endif
     {IMGFMT_VAAPI,           AV_PIX_FMT_VAAPI_VLD},
 
     {0, AV_PIX_FMT_NONE}
