@@ -32,6 +32,7 @@
 
 #define VO_EVENT_EXPOSE 1
 #define VO_EVENT_RESIZE 2
+#define VO_EVENT_ICC_PROFILE_PATH_CHANGED 4
 
 enum mp_voctrl {
     /* signal a device reset seek */
@@ -53,8 +54,9 @@ enum mp_voctrl {
     /* for hardware decoding */
     VOCTRL_GET_HWDEC_INFO,              // struct mp_hwdec_info*
 
-    VOCTRL_NEWFRAME,
-    VOCTRL_SKIPFRAME,
+    // Redraw the image previously passed to draw_image() (basically, repeat
+    // the previous draw_image call). If this is handled, the OSD should also
+    // be updated and redrawn.
     VOCTRL_REDRAW_FRAME,
 
     VOCTRL_ONTOP,
@@ -69,17 +71,22 @@ enum mp_voctrl {
     VOCTRL_SET_DEINTERLACE,
     VOCTRL_GET_DEINTERLACE,
 
-    VOCTRL_UPDATE_SCREENINFO,
     VOCTRL_WINDOW_TO_OSD_COORDS,        // float[2] (x/y)
     VOCTRL_GET_WINDOW_SIZE,             // int[2] (w/h)
     VOCTRL_SET_WINDOW_SIZE,             // int[2] (w/h)
 
-    VOCTRL_SET_YUV_COLORSPACE,          // struct mp_csp_details*
-    VOCTRL_GET_YUV_COLORSPACE,          // struct mp_csp_details*
+    // The VO is supposed to set  "known" fields, and leave the others
+    // untouched or set to 0.
+    // imgfmt/w/h/d_w/d_h can be omitted for convenience.
+    VOCTRL_GET_COLORSPACE,              // struct mp_image_params*
 
     VOCTRL_SCREENSHOT,                  // struct voctrl_screenshot_args*
 
     VOCTRL_SET_COMMAND_LINE,            // char**
+
+    VOCTRL_GET_ICC_PROFILE_PATH,        // char**
+
+    VOCTRL_GET_PREF_DEINT,              // int*
 };
 
 // VOCTRL_SET_EQUALIZER
@@ -107,7 +114,7 @@ struct voctrl_screenshot_args {
     // The caller has to free the image with talloc_free().
     // It is not specified whether the image data is a copy or references the
     // image data directly.
-    // Is never NULL. (Failure has to be indicated by returning VO_FALSE.)
+    // Can be NULL on failure.
     struct mp_image *out_image;
     // Whether the VO rendered OSD/subtitles into out_image
     bool has_osd;
@@ -115,29 +122,33 @@ struct voctrl_screenshot_args {
 
 #define VO_TRUE         true
 #define VO_FALSE        false
-#define VO_ERROR	-1
-#define VO_NOTAVAIL	-2
-#define VO_NOTIMPL	-3
+#define VO_ERROR        -1
+#define VO_NOTAVAIL     -2
+#define VO_NOTIMPL      -3
 
-#define VOFLAG_FLIPPING		0x08
-#define VOFLAG_HIDDEN		0x10  //< Use to create a hidden window
-#define VOFLAG_STEREO		0x20  //< Use to create a stereo-capable window
+#define VOFLAG_FLIPPING         0x08
+#define VOFLAG_HIDDEN           0x10  //< Use to create a hidden window
+#define VOFLAG_STEREO           0x20  //< Use to create a stereo-capable window
 #define VOFLAG_GL_DEBUG         0x40  // Hint to request debug OpenGL context
 #define VOFLAG_ALPHA            0x80  // Hint to request alpha framebuffer
 
+// VO does handle mp_image_params.rotate in 90 degree steps
+#define VO_CAP_ROTATE90 1
+
+#define VO_MAX_QUEUE 5
+
 struct vo;
 struct osd_state;
 struct mp_image;
 struct mp_image_params;
 
 struct vo_driver {
-    // Driver buffers or adds (deinterlace) frames and will keep track
-    // of pts values itself
-    bool buffer_frames;
-
     // Encoding functionality, which can be invoked via --o only.
     bool encode;
 
+    // VO_CAP_* bits
+    int caps;
+
     const char *name;
     const char *description;
 
@@ -154,21 +165,21 @@ struct vo_driver {
     int (*query_format)(struct vo *vo, uint32_t format);
 
     /*
-     * Initialize or reconfigure the display driver.
-     *   width,height: image source size
-     *   d_width,d_height: requested window size, just a hint
-     *   flags: combination of VOFLAG_ values
-     *   title: window title, if available
-     *   format: fourcc of pixel format
-     * returns : zero on successful initialization, non-zero on error.
+     * Optional. Can be used to convert the input image into something VO
+     * internal, such as GPU surfaces. Ownership of mpi is passed to the
+     * function, and the returned image is owned by the caller.
+     * The following guarantees are given:
+     * - mpi has the format with which the VO was configured
+     * - the returned image can be arbitrary, and the VO merely manages its
+     *   lifetime
+     * - images passed to draw_image are always passed through this function
+     * - the maximum number of images kept alive is not over vo->max_video_queue
+     * - if vo->max_video_queue is large enough, some images may be buffered ahead
      */
-    int (*config)(struct vo *vo, uint32_t width, uint32_t height,
-                  uint32_t d_width, uint32_t d_height, uint32_t flags,
-                  uint32_t format);
+    struct mp_image *(*filter_image)(struct vo *vo, struct mp_image *mpi);
 
     /*
-     * Initialize or reconfigure the display driver. Alternative to config(),
-     * and can carry more image parameters.
+     * Initialize or reconfigure the display driver.
      *   params: video parameters, like pixel format and frame size
      *   flags: combination of VOFLAG_ values
      * returns: < 0 on error, >= 0 on success
@@ -180,21 +191,15 @@ struct vo_driver {
      */
     int (*control)(struct vo *vo, uint32_t request, void *data);
 
+    /*
+     * Render the given frame to the VO's backbuffer. This operation will be
+     * followed by a draw_osd and a flip_page[_timed] call.
+     * mpi belongs to the VO; the VO must free it eventually.
+     *
+     * This also should draw the OSD.
+     */
     void (*draw_image)(struct vo *vo, struct mp_image *mpi);
 
-    /*
-     * Get extra frames from the VO, such as those added by VDPAU
-     * deinterlace. Preparing the next such frame if any could be done
-     * automatically by the VO after a previous flip_page(), but having
-     * it as a separate step seems to allow making code more robust.
-     */
-    void (*get_buffered_frame)(struct vo *vo, bool eof);
-
-    /*
-     * Draws OSD to the screen buffer
-     */
-    void (*draw_osd)(struct vo *vo, struct osd_state *osd);
-
     /*
      * Blit/Flip buffer to the screen. Must be called after each frame!
      * pts_us is the frame presentation time, linked to mp_time_us().
@@ -221,28 +226,24 @@ struct vo_driver {
 };
 
 struct vo {
-    struct {
-        struct mp_log *log; // Using "[vo]" as prefix
-    } vo_log;
     struct mp_log *log; // Using e.g. "[vo/vdpau]" as prefix
     int config_ok;      // Last config call was successful?
-    int config_count;   // Total number of successful config calls
     struct mp_image_params *params; // Configured parameters (as in vo_reconfig)
 
     bool probing;
 
     bool untimed;       // non-interactive, don't do sleep calls in playloop
 
-    bool frame_loaded;  // Is there a next frame the VO could flip to?
-    struct mp_image *waiting_mpi;
-    double next_pts;    // pts value of the next frame if any
-    double next_pts2;   // optional pts of frame after that
     bool want_redraw;   // visible frame wrong (window resize), needs refresh
-    bool redrawing;     // between redrawing frame and flipping it
     bool hasframe;      // >= 1 frame has been drawn, so redraw is possible
     double wakeup_period; // if > 0, this sets the maximum wakeup period for event polling
 
     double flip_queue_offset; // queue flip events at most this much in advance
+    int max_video_queue; // queue this many decoded video frames (<=VO_MAX_QUEUE)
+
+    // Frames to display; the next (i.e. oldest, lowest PTS) image has index 0.
+    struct mp_image *video_queue[VO_MAX_QUEUE];
+    int num_video_queue;
 
     const struct vo_driver *driver;
     void *priv;
@@ -254,26 +255,13 @@ struct vo {
     struct vo_wayland_state *wayland;
     struct encode_lavc_context *encode_lavc_ctx;
     struct input_ctx *input_ctx;
+    struct osd_state *osd;
     int event_fd;  // check_events() should be called when this has input
-    int registered_fd;  // set to event_fd when registered in input system
 
-    // requested position/resolution (usually window position/window size)
-    int dx;
-    int dy;
+    // current window state
     int dwidth;
     int dheight;
-
-    int xinerama_x;
-    int xinerama_y;
-
-    struct aspect_data {
-        float monitor_par; // out of screen size or from options
-        int orgw; // real width
-        int orgh; // real height
-        int prew; // prescaled width
-        int preh; // prescaled height
-        float par; // pixel aspect ratio out of orgw/orgh and prew/preh
-    } aspdat;
+    float monitor_par;
 
     char *window_title;
 };
@@ -281,27 +269,25 @@ struct vo {
 struct mpv_global;
 struct vo *init_best_video_out(struct mpv_global *global,
                                struct input_ctx *input_ctx,
+                               struct osd_state *osd,
                                struct encode_lavc_context *encode_lavc_ctx);
 int vo_reconfig(struct vo *vo, struct mp_image_params *p, int flags);
 
 int vo_control(struct vo *vo, uint32_t request, void *data);
 void vo_queue_image(struct vo *vo, struct mp_image *mpi);
-int vo_redraw_frame(struct vo *vo);
 bool vo_get_want_redraw(struct vo *vo);
-int vo_get_buffered_frame(struct vo *vo, bool eof);
-void vo_skip_frame(struct vo *vo);
+bool vo_has_next_frame(struct vo *vo, bool eof);
+double vo_get_next_pts(struct vo *vo, int index);
+bool vo_needs_new_image(struct vo *vo);
 void vo_new_frame_imminent(struct vo *vo);
-void vo_draw_osd(struct vo *vo, struct osd_state *osd);
 void vo_flip_page(struct vo *vo, int64_t pts_us, int duration);
+void vo_redraw(struct vo *vo);
 void vo_check_events(struct vo *vo);
 void vo_seek_reset(struct vo *vo);
 void vo_destroy(struct vo *vo);
 
 const char *vo_get_window_title(struct vo *vo);
 
-// NULL terminated array of all drivers
-extern const struct vo_driver *video_out_drivers[];
-
 struct mp_keymap {
   int from;
   int to;
@@ -309,6 +295,7 @@ struct mp_keymap {
 int lookup_keymap_table(const struct mp_keymap *map, int key);
 
 void vo_mouse_movement(struct vo *vo, int posx, int posy);
+void vo_drop_files(struct vo *vo, int num_files, char **files);
 
 struct mp_osd_res;
 void vo_get_src_dst_rects(struct vo *vo, struct mp_rect *out_src,
