@@ -4,6 +4,8 @@
 #include <stdbool.h>
 #include <inttypes.h>
 
+#include <pthread.h>
+
 #include <vdpau/vdpau.h>
 #include <vdpau/vdpau_x11.h>
 
@@ -35,38 +37,53 @@ struct vdp_functions {
 // Shared state. Objects created from different VdpDevices are often (always?)
 // incompatible to each other, so all code must use a shared VdpDevice.
 struct mp_vdpau_ctx {
-    struct vdp_functions *vdp;
+    struct mp_log *log;
+    struct vo_x11_state *x11;
+
+    // These are mostly immutable, except on preemption. We don't really care
+    // to synchronize the preemption case fully correctly, because it's an
+    // extremely obscure corner case, and basically a vdpau API design bug.
+    // What we do will sort-of work anyway (no memory errors are possible).
+    struct vdp_functions vdp;
     VdpGetProcAddress *get_proc_address;
     VdpDevice vdp_device;
+
+    pthread_mutex_t preempt_lock;
     bool is_preempted;                  // set to true during unavailability
     uint64_t preemption_counter;        // incremented after _restoring_
-
     bool preemption_user_notified;
     double last_preemption_retry_fail;
 
-    struct vo_x11_state *x11;
-
     // Surface pool
+    pthread_mutex_t pool_lock;
     struct surface_entry {
         VdpVideoSurface surface;
-        int fmt, w, h;
+        VdpOutputSurface osurface;
+        bool allocated;
+        int w, h;
+        VdpRGBAFormat rgb_format;
         VdpChromaType chroma;
+        bool rgb;
         bool in_use;
     } video_surfaces[MAX_VIDEO_SURFACES];
-
-    struct mp_log *log;
 };
 
 struct mp_vdpau_ctx *mp_vdpau_create_device_x11(struct mp_log *log,
                                                 struct vo_x11_state *x11);
 void mp_vdpau_destroy(struct mp_vdpau_ctx *ctx);
 
-bool mp_vdpau_status_ok(struct mp_vdpau_ctx *ctx);
+int mp_vdpau_handle_preemption(struct mp_vdpau_ctx *ctx, uint64_t *counter);
 
-struct mp_image *mp_vdpau_get_video_surface(struct mp_vdpau_ctx *ctx, int fmt,
+struct mp_image *mp_vdpau_get_video_surface(struct mp_vdpau_ctx *ctx,
                                             VdpChromaType chroma, int w, int h);
 
 bool mp_vdpau_get_format(int imgfmt, VdpChromaType *out_chroma_type,
                          VdpYCbCrFormat *out_pixel_format);
+bool mp_vdpau_get_rgb_format(int imgfmt, VdpRGBAFormat *out_rgba_format);
+
+struct mp_image *mp_vdpau_upload_video_surface(struct mp_vdpau_ctx *ctx,
+                                               struct mp_image *mpi);
+
+bool mp_vdpau_guess_if_emulated(struct mp_vdpau_ctx *ctx);
 
 #endif
